/*
** Copyright (C) 1993-2013 Mark B. Hanson (mbh@panix.com)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

static const char *const file_id =
	"@(#)$Id: text.C 1344 2013-04-05 21:18:39Z mark $";

#include "global.h"

#include "font.h"
#include "util.h"
#include "xsc.h"

#include "text.h"


Text::Text(void)
{
    //fprintf(stderr, "Text::Text()\n");
    set_scale(6.0);
    set_gc(fetch_gc(GC_BLUE));
} // Text::Text


Text::~Text(void)
{
    //fprintf(stderr, "Text::~Text()\n");
} // Text::~Text


void
Text::set_position(const float xpos, const float ypos)
{
    if (xpos == -1) {
	const float charwidth = size / 6.0;
	float width = 0;

	for (size_t i = 0; i < length; i++) {
	    if ((message[i] >= '0' && message[i] <= '9') ||
		    (message[i] >= 'A' && message[i] <= 'Z') ||
		    (message[i] >= 'a' && message[i] <= 'z') ||
		    message[i] == ' ') {
		width += charwidth;
	    } else if (message[i] == ',') {
		width += charwidth / 2.0;
	    } else {
		continue;
	    }
	}

	x = wwidth2 + (charwidth / 2.0) - (width / 2.0);
    } else {
	x = xpos;
    }

    y = ypos;
} // Text::set_position


void
Text::render(const bool ink)
{
    const float charwidth = size / 6.0;
    const float savex = x;
    float width;
    int offset;

    for (size_t i = 0; i < length; i++) {
	width = charwidth;
	if (message[i] >= '0' && message[i] <= '9') {
	    offset = font_digit_offset + message[i] - '0';
	} else if (message[i] >= 'A' && message[i] <= 'Z') {
	    offset = font_upper_offset + message[i] - 'A';
	} else if (message[i] >= 'a' && message[i] <= 'z') {
	    offset = font_lower_offset + message[i] - 'a';
	} else if (message[i] == ',') {
	    offset = font_comma_offset;
	    width /= 2.0;
	} else if (message[i] == ' ') {
	    x += width;
	    continue;
	} else {
	    continue;
	}

	set_points(romans_points[offset], romans_count[offset]);
	set_xpoints();
	paint_points(ink);

	x += width;
    }

    x = savex;
} // Text::render
